"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModifiedFile = void 0;
const fs_1 = require("fs");
const utils_1 = require("./utils");
const git_utils_1 = require("./git-utils");
const unknown_1 = require("./unknown");
const nullable_1 = require("@softwareventures/nullable");
const path_1 = require("path");
const execa = require("execa");
class ModifiedFile {
    constructor({ fullPath, gitDirectoryParent, base, head, selectedFormatter }) {
        /**
         * The final file contents, after we've run the formatter
         */
        this.formattedFileContents = null;
        /**
         * The calculated character ranges which have been modified
         * within this file
         */
        this.modifiedCharacterRanges = [];
        this.fullPath = fullPath;
        this.pathInGit = (0, path_1.relative)(gitDirectoryParent, fullPath).split(path_1.sep).join(path_1.posix.sep);
        this.gitDirectoryParent = gitDirectoryParent;
        this.base = base;
        this.head = head;
        this.selectedFormatter = selectedFormatter;
        this.fileContents =
            head === git_utils_1.workingTree
                ? (0, fs_1.readFileSync)(this.fullPath, "utf8")
                : head === git_utils_1.index
                    ? execa.sync("git", ["show", `:0:${this.pathInGit}`], {
                        cwd: gitDirectoryParent,
                        stripFinalNewline: false
                    }).stdout
                    : execa.sync("git", ["show", `${head}:${this.pathInGit}`], {
                        cwd: gitDirectoryParent,
                        stripFinalNewline: false
                    }).stdout;
        this.formatterConfig = this.selectedFormatter.resolveConfig(this.fullPath);
    }
    /**
     * Return true if the whole file has already been formatted appropriately based on
     * the resolved formatter config. We can use this as a check to skip unnecessary work.
     */
    isAlreadyFormatted() {
        return this.selectedFormatter.isAlreadyFormatted(this.fileContents, this.formatterConfig);
    }
    /**
     * Run the formatters check mode on the given ranges and return true if they are all
     * already formatted appropriately based on the resolved formatter config.
     */
    hasValidFormattingForCharacterRanges() {
        return this.selectedFormatter.checkFormattingOfRanges(this.fileContents, this.formatterConfig, this.modifiedCharacterRanges);
    }
    /**
     * Run the formatter on the file contents and store the result
     */
    formatCharacterRangesWithinContents() {
        this.formattedFileContents = this.selectedFormatter.formatRanges(this.fileContents, this.formatterConfig, this.modifiedCharacterRanges);
    }
    /**
     * Return true if the formatted file contents are different to
     * what was originally resolved from disk.
     */
    shouldContentsBeUpdatedOnDisk() {
        return this.fileContents !== this.formattedFileContents;
    }
    /**
     * Write the updated file contents back to disk.
     */
    updateFileOnDisk() {
        var _a;
        if (this.head === git_utils_1.index) {
            const hash = execa.sync("git", ["hash-object", "-w", "--path", this.fullPath, "--stdin"], {
                cwd: this.gitDirectoryParent,
                input: (0, nullable_1.notNull)(this.formattedFileContents)
            }).stdout;
            const mode = (_a = execa
                .sync("git", ["ls-files", "--stage", "--", this.fullPath], {
                cwd: this.gitDirectoryParent
            })
                .stdout.split(" ")) === null || _a === void 0 ? void 0 : _a[0];
            if (mode == null) {
                throw new Error("Can't find file in git index");
            }
            execa.sync("git", [
                "update-index",
                "--add",
                "--replace",
                "--cacheinfo",
                `100644,${hash},${this.pathInGit}`
            ], { cwd: this.gitDirectoryParent });
        }
        else {
            (0, fs_1.writeFileSync)(this.fullPath, (0, nullable_1.notNull)(this.formattedFileContents));
        }
    }
    /**
     * We handle errors locally within this method to allow for
     * more granular feedback within the main() function of the
     * library.
     */
    calculateModifiedCharacterRanges() {
        try {
            /**
             * Extract line change data from the git diff results.
             */
            const diff = (0, git_utils_1.getDiffForFile)(this.gitDirectoryParent, this.fullPath, this.base, this.head);
            const lineChangeData = (0, utils_1.extractLineChangeData)(diff);
            /**
             * Convert the line change data into character data.
             */
            this.modifiedCharacterRanges = (0, utils_1.calculateCharacterRangesFromLineChanges)(lineChangeData, this.fileContents);
            return { err: null };
        }
        catch (err) {
            return { err: (0, unknown_1.assertInstanceOf)(err, Error) };
        }
    }
}
exports.ModifiedFile = ModifiedFile;
//# sourceMappingURL=modified-file.js.map