"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.preciseFormatterPrettier = void 0;
const fs = require("fs");
const ignore_1 = require("ignore");
const path_1 = require("path");
const prettier_1 = require("prettier");
let PRETTIER_SUPPORTED_FILE_EXTENSIONS = [];
(0, prettier_1.getSupportInfo)().languages.forEach(language => {
    var _a;
    PRETTIER_SUPPORTED_FILE_EXTENSIONS = [
        ...PRETTIER_SUPPORTED_FILE_EXTENSIONS,
        ...((_a = language.extensions) !== null && _a !== void 0 ? _a : [])
    ];
});
exports.preciseFormatterPrettier = {
    /**
     * Resolve the relevant prettier config for the given
     * modified file path.
     */
    resolveConfig(modifiedFilePath) {
        return Object.assign(Object.assign({}, prettier_1.resolveConfig.sync(modifiedFilePath, {
            useCache: false
        })), { filepath: modifiedFilePath });
    },
    /**
     * Return true if the whole file has already been formatted appropriately based on
     * the resolved prettier config. We can use this as a check to skip unnecessary work.
     */
    isAlreadyFormatted(fileContents, config) {
        return (0, prettier_1.check)(fileContents, Object.assign({}, config));
    },
    /**
     * Run prettier's check mode on the given ranges and return true if they are all
     * already formatted appropriately based on the given prettier config.
     */
    checkFormattingOfRanges(fileContents, config, characterRanges) {
        let formattedContents = fileContents;
        return characterRanges.every(characterRange => {
            return (0, prettier_1.check)(formattedContents, Object.assign(Object.assign({}, config), {
                rangeStart: characterRange.rangeStart,
                rangeEnd: characterRange.rangeEnd
            }));
        });
    },
    /**
     * Run prettier on each character range pair given, and apply the
     * difference as a patch to the original contents using an implementation
     * of the Myer's diff algorithm.
     */
    formatRanges(fileContents, config, characterRanges) {
        // Start from the last character range and work backwards so that
        // we don't have to update character ranges to account for the changes
        // we've already made.
        return characterRanges.reduceRight((fileContents, { rangeStart, rangeEnd }) => (0, prettier_1.format)(fileContents, Object.assign(Object.assign({}, config), { rangeStart, rangeEnd })), fileContents);
    },
    /**
     * Generate a predicate function which will return true if the filename
     * is not excluded via a .prettierignore file.
     */
    generateIgnoreFilePredicate(workingDirectory) {
        const prettierIgnoreFilePath = (0, path_1.join)(workingDirectory, ".prettierignore");
        /**
         * If there is no .prettierignore file present, simply always return true
         * from the predicate
         */
        if (!fs.existsSync(prettierIgnoreFilePath)) {
            return () => true;
        }
        /**
         * Use "ignore"'s createFilter() method to create a predicate
         */
        const prettierIgnoreFileContents = fs.readFileSync(prettierIgnoreFilePath, "utf8");
        return (0, ignore_1.default)().add(prettierIgnoreFileContents).createFilter();
    },
    /**
     * Return true if prettier supports the file extension of the given
     * filename.
     */
    hasSupportedFileExtension(filename) {
        const fileExtension = (0, path_1.extname)(filename);
        return PRETTIER_SUPPORTED_FILE_EXTENSIONS.includes(fileExtension);
    }
};
//# sourceMappingURL=prettier.js.map